import mongoose from 'mongoose';
import emailer from './emailer';

const LANG_MAX_AGE = 20 * 365 * 24 * 3600 * 1000;
const mAvailableLangs = [];
const Schema = mongoose.Schema;
const langSchema = new Schema({
    lang: String,
    displayName: String,
});
let mLogger;

export const Lang = mongoose.model('lang', langSchema);

export const getLogger = () => mLogger;

function selectDialect(acceptLanguages, defaultLang) {
    // ['ca', 'en-UK;q=0.8', 'fr;q=0.6', 'es-ES;q=0.4', 'es;q=0.2']
    // Look for a different dialect: maybe we have en-US or fr-CA or ex-MX
    for (let i = 0; i < acceptLanguages.length; i += 1) {
        let lang = acceptLanguages[i].split(';')[0];
        lang = lang.indexOf('-') < 0 ? lang : lang.split('-')[0];
        for (let l = 0; l < mAvailableLangs.length; l += 1) {
            if (mAvailableLangs[l].indexOf(lang) === 0) {
                return mAvailableLangs[l];
            }
        }
    }
    // No language found, return default
    return defaultLang;
}

function selectLang(acceptLanguages, defaultLang) {
    // ['ca', 'en-UK;q=0.8', 'fr;q=0.6', 'es-ES;q=0.4', 'es;q=0.2']
    for (let i = 0; i < acceptLanguages.length; i += 1) {
        const lang = acceptLanguages[i].split(';')[0];
        if (mAvailableLangs.indexOf(lang.toLowerCase()) >= 0) {
            return lang.toLowerCase();
        }
    }

    // We only get here if no language has been found
    return selectDialect(acceptLanguages, defaultLang);
}

function appendLanguage(lang) {
    let isNew = false;
    if (mAvailableLangs.indexOf(lang.toLowerCase()) < 0) {
        isNew = true;
        mAvailableLangs.push(lang.toLowerCase());
    }

    return isNew;
}

function initLanguages() {
    return Lang.find().exec()
        .then((langData) => {
            for (let l = 0; l < langData.length; l += 1) {
                appendLanguage(langData[l].lang);
            }
        })
        .catch((langErr) => {
            mLogger.error('Error selecting language:', langErr);
            return Promise.reject(langErr);
        });
}

/**
 * Library entry point. This will find all locales and return the middleware
 * to get language from request.
 *
 * @param options {Object} { defaultLang, maxAge }
 *  - defaultLang: Default language to use (by default 'en')
 *  - maxAge: maxAge for 'lang' cookie
 *  - isSetCookie: use 'lang' cookie to store language settings (defaults true)
 *  - logger: logger function to send log messages (must implement error, info and waring methods)
 *  - email: Object with { transport, from, to } options. If provided, will be used to communicate
 *    missing translations
 *    - transport: must implement sendMail(message, callback)
 *    - from: the text that will appear in the 'from' field when a message is sent
 *    - to: the receiver of the emails
 * @param callback {Function} (optional) callback function which is called with a Promise.
 *  The promise will be resolved once all available languages have been loaded from database
 */
export default (options, callback) => {
    const { defaultLang, maxAge, isSetCookie, logger, email } = Object.assign({
        defaultLang: 'en',
        maxAge: LANG_MAX_AGE,
        isSetCookie: true,
        logger: {
            log: () => { },
            error: () => { },
            warning: () => { },
            info: () => { },
        },
    }, options || {});

    const middleWare = (req, res, next) => {
        /* eslint-disable no-param-reassign */
        if (req.cookies && req.cookies.lang) {
            req.lang = req.cookies.lang;
        } else {
            // Ex: 'ca,en-UK;q=0.8,fr;q=0.6,es-ES;q=0.4,es;q=0.2'
            const acceptLanguage = req.headers['accept-language'] || '';
            req.lang = acceptLanguage ? selectLang(acceptLanguage.split(','), defaultLang) : defaultLang;
            if (isSetCookie) {
                res.cookie('lang', req.lang, {
                    maxAge,
                    httpOnly: false,
                });
            }
        }
        /* eslint-enable no-param-reassign */
        next();
    };

    mLogger = logger;

    if (email) {
        emailer(email);
    }

    (callback || (() => { }))(initLanguages());

    return middleWare;
};

export const getAvailableLangs = data => Lang.find(data).sort({ lang: 1 }).exec();

export const addNewLanguage = (lang, displayName) => {
    if (appendLanguage(lang)) {
        Lang.findOneAndUpdate(
            { lang },
            { lang, displayName: displayName || '' },
            { new: true, upsert: true },
        ).exec();
    }
};

