import mongoose from 'mongoose';
import { expandPath, getObjectByPath, updateObjectByPath } from 'jsobjects';
import { addNewLanguage, getLogger } from './lang-mongoose';
import { fromTemplate } from './emailer';

const logger = getLogger();
const Schema = mongoose.Schema;
const localeSchema = new Schema({
    strings: [
        {
            _id: false,
            lang: String,
            text: String,
            extra: String,
        },
    ],
    refs: [Schema.Types.ObjectId],
});
const localeTypesSchema = new Schema({
    type: String,
});

// eslint-disable-next-line func-names
localeTypesSchema.statics.findAndModify = function (query, sort, update, opts, callback) {
    const cb = callback || (() => { });
    try {
        const result = this.collection.findAndModify(query || {}, sort || [], update || {}, opts);
        cb(null, result);
        return Promise.resolve(result);
    } catch (err) {
        cb(err);
        return Promise.reject(err);
    }
};

export const Locale = mongoose.model('locale', localeSchema);
export const LocaleTypes = mongoose.model('localetypes', localeTypesSchema);

function getTranslatableKeys(schema) {
    const paths = schema.paths;
    const pathKeys = Object.keys(paths);
    const translatableKeys = [];
    pathKeys.forEach((key) => {
        const p = paths[key];
        if (p instanceof Schema.Types.ObjectId && p.options && p.options.ref === 'locale') {
            translatableKeys.push(key);
        } else if (p instanceof Schema.Types.DocumentArray) {
            const stk = getTranslatableKeys(p.schema);
            stk.forEach(subKey => translatableKeys.push(`${key}.*.${subKey}`));
        }
    });
    return translatableKeys;
}

function updateLocale(locale, newLocales) {
    let isModified = false;
    let mutLocale = locale;
    if (!mutLocale) {
        isModified = true;
        mutLocale = new Locale({
            strings: [],
            refs: [],
        });
    }
    mutLocale.strings = mutLocale.strings || [];
    mutLocale.strings.forEach((mlStr, mIdx) => {
        newLocales.strings.some((nlStr, nIdx) => {
            let isFound = false;
            if (nlStr.lang === mlStr.lang && nlStr.text !== mlStr.text) {
                isModified = true;
                mutLocale.strings[mIdx].text = nlStr.text;
                newLocales.strings.splice(nIdx, 1);
                isFound = true;
            } else if (nlStr.lang === mlStr.lang) {
                newLocales.strings.splice(nIdx, 1);
                isFound = true;
            }
            return isFound;
        });
    });
    if (newLocales.strings.length) {
        mutLocale.strings.push(...newLocales.strings);
        isModified = true;
    }

    return isModified ? mutLocale.save() : mutLocale;
}

function saveLocales(docData, field, replacements) {
    const detectedLangs = {};
    let promise;
    let newLocales;

    try {
        newLocales = getObjectByPath(docData, field, replacements);
    } catch (e) {
        logger.error(e);
    }

    if (newLocales) {
        if (newLocales._id) {
            promise = Locale.findById(newLocales._id)
                .exec()
                .then(locale => updateLocale(locale, newLocales));
        } else {
            promise = new Locale({
                strings: newLocales.strings,
                refs: docData._id ? [docData._id] : [],
            })
                .save();
        }

        // Actualitzar els llenguatges disponibles
        Object.keys(detectedLangs).forEach((lang) => {
            addNewLanguage(lang);
        });
    } else {
        promise = Promise.resolve();
    }

    return promise
        .then((locale) => {
            if (locale) {
                updateObjectByPath(docData, field, locale._id, replacements);
            }
            return locale;
        });
}

function processLocales(docData, translatableKeys) {
    const promises = [];
    translatableKeys.forEach((field) => {
        if (/\.\*\./i.test(field)) {
            expandPath(docData, field).forEach((expandedField) => {
                promises.push(saveLocales(docData, expandedField));
            });
        } else {
            promises.push(saveLocales(docData, field));
        }
    });
    return Promise.all(promises);
}

function saveLocalized(docData, Model, translatableKeys) {
    return processLocales(docData, translatableKeys)
        .then((locales) => {
            let promise;
            if (docData._id) {
                promise = Model.findByIdAndUpdate(docData._id, { $set: docData }, {
                    upsert: true,
                    new: true,
                }).exec();
            } else {
                promise = new Model(docData).save();
            }

            return promise.then(doc => ({ doc, locales }));
        })
        .then(({ doc, locales }) => {
            const promises = [];
            locales.forEach((locale) => {
                if (locale && locale.refs.indexOf(doc._id) < 0) {
                    locale.refs.push(doc._id);
                    promises.push(locale.save());
                } else {
                    promises.push(Promise.resolve(locale));
                }
            });
            return Promise.all(promises).then(() => doc);
        });
}

export const findByRef = (ref, lang) => {
    let query;
    if (lang) {
        query = Locale.aggregate(
            [
                { $match: { refs: { $in: [ref] } } },
                {
                    $project: {
                        strings: {
                            $filter: {
                                input: '$strings',
                                as: 'string',
                                cond: { $eq: ['$$string.lang', lang] },
                            },
                        },
                    },
                },
            ])
            .exec();
    } else {
        query = Locale.find({ refs: { $in: [ref] } }).lean().exec();
    }

    return query;
};

export const findByType = (type, lang) => {
    let promise;
    if (lang && lang !== 'en') {
        promise = LocaleTypes
            .find({ type })
            .lean()
            .limit(1)
            .then(types => (
                types.length ?
                    Locale.aggregate(
                        [
                            { $match: { refs: { $eq: types[0]._id } } },
                            { $unwind: '$strings' },
                            { $match: { 'strings.lang': { $in: ['--', lang] } } },
                            { $group: { _id: '$_id', strings: { $push: '$strings.text' } } },
                        ])
                    :
                    Promise.resolve([])
            ))
            .then((locales) => {
                const translations = {};
                (locales || []).forEach((locale) => {
                    if (locale.strings.length > 1) {
                        translations[locale.strings[0]] = locale.strings[1];
                    }
                });

                return translations;
            });
    } else {
        promise = Promise.resolve({});
    }

    return promise;
};

export const missing = ({ type, text, lang, extra }) => {
    let missingPromise;
    if (lang !== 'en') {
        // find or create: http://stackoverflow.com/a/16362833/4025963
        missingPromise = LocaleTypes
            .findAndModify({ type }, [], { $setOnInsert: { type } }, { new: true, upsert: true })
            .then(typedoc => typedoc.value)
            .then(typeDoc =>
                Locale.find({
                    refs: typeDoc._id,
                    strings: {
                        $elemMatch: { text },
                    },
                })
                    .exec()
                    .then(locales => ({ locales, typeDoc })))
            .then(({ locales, typeDoc }) => {
                let promise;
                let isAlreadyPresent = false;

                if (locales && locales.length) {
                    locales[0].strings.forEach((str) => {
                        if (str.lang === lang) {
                            isAlreadyPresent = true;
                        }
                        return isAlreadyPresent;
                    });

                    if (!isAlreadyPresent) {
                        locales[0].strings.push({
                            lang, text: '',
                        });
                        promise = locales[0].save();
                        logger.info(`Inserted translation for "${text}" - "${lang}"`);
                        if (process.env.NODE_ENV === 'production') {
                            fromTemplate('missingTranslation', {}, { type, text, lang, extra })
                                .catch(logger.error);
                        }
                    } else {
                        promise = Promise.resolve();
                    }
                } else {
                    promise = new Locale({
                        refs: [typeDoc._id],
                        strings: [
                            { text, lang: '--' },
                            { text: '', lang, extra },
                        ],
                    }).save();
                    logger.info(`Inserted new translation for "${text}" - "${lang}"`);
                    if (process.env.NODE_ENV === 'production') {
                        fromTemplate('missingTranslation', {}, { type, text, lang, extra })
                            .catch(logger.error);
                    }
                }
                return promise;
            });
    } else {
        missingPromise = Promise.resolve();
    }

    return missingPromise;
};

export const remove = refId =>
    Locale.find({ refs: refId })
        .exec()
        .then((locales) => {
            const promises = [];
            locales.forEach((locale) => {
                locale.refs.splice(locale.refs.indexOf(refId), 1);
                if (locale.refs.length) {
                    promises.push(locale.save());
                } else {
                    promises.push(locale.remove());
                }
            });

            return Promise.all(promises);
        });

export const localizableModel = (name, schema, collection, skipInit) => {
    const Model = mongoose.model(name, schema, collection, skipInit);
    const translatableKeys = getTranslatableKeys(schema);

    Model.findLocales = (query, lang) => Model.find(query)
        .limit(1)
        .lean()
        .exec()
        .then(docs => findByRef(docs[0]._id, lang));

    Model.findLocalized = (query, lang) => Model.find(query)
        .lean()
        .exec()
        .then((docs) => {
            const searchLocales = [];
            const kLocales = {};
            // Per cada document, busquem els locales que ténen referenciat l'_id del doc
            (docs || []).forEach((doc, docIdx) => {
                const findByRefPromise = findByRef(doc._id, lang)
                    .then((locales) => {
                        kLocales[docIdx] = {};
                        locales.forEach((locale) => {
                            kLocales[docIdx][locale._id.toString()] = locale;
                        });
                        return kLocales;
                    });
                searchLocales.push(findByRefPromise);
            });
            return Promise.all(searchLocales).then(() => ({ kLocales, docs }));
        })
        .then(({ kLocales, docs }) => {
            docs.forEach((doc, dIdx) => {
                // Posem el text corresponent a cada translatableKey
                translatableKeys.forEach((key) => {
                    if (/\.\*/ig.test(key)) {
                        expandPath(doc, key, (path, locId) => {
                            if (locId) {
                                const locale = kLocales[dIdx][locId.toString()];
                                if (lang) {
                                    updateObjectByPath(docs[dIdx], path, (locale.strings[0] || { text: '' }).text);
                                } else {
                                    updateObjectByPath(docs[dIdx], path, locale);
                                }
                            }
                        });
                    } else {
                        try {
                            const locId = getObjectByPath(doc, key);
                            if (locId) {
                                const locale = kLocales[dIdx][locId.toString()];
                                if (lang) {
                                    updateObjectByPath(docs[dIdx], key, (locale.strings[0] || { text: '' }).text);
                                } else {
                                    updateObjectByPath(docs[dIdx], key, locale);
                                }
                            }
                        } catch (e) {
                            logger.error(e);
                        }
                    }
                });
            });

            return docs;
        });

    Model.saveLocalized = (docData, itemId) => {
        if (itemId && docData._id !== itemId) {
            docData._id = itemId; // eslint-disable-line no-param-reassign
        }
        return saveLocalized(docData, Model, translatableKeys);
    };

    schema.pre('remove', function (next) { // eslint-disable-line func-names
        remove(this._id).then(() => next(), next);
    });
    return Model;
};

