import handlebars from 'handlebars';
import moment from 'moment';
import { Locale, missing } from './locales';
import { hashCode } from './util';
import { getLogger } from './lang-mongoose';

const logger = getLogger();
const cached = {};
const hashes = [];
const CACHE_EXPIRE_MS = 60 * 3600 * 1000;
const CACHE_MAX_KEYS = 10000;
let cachedCount = 0;

function formatDate(date, lang, format) {
    return moment(date).locale(lang).format(format);
}

export const registerHelpers = (helpers) => {
    Object.keys(helpers).forEach(h => handlebars.registerHelper(h, helpers[h]));
};

registerHelpers({
    formatDate: (date, lang) => formatDate(date, lang, 'LL'),
    formatDateTime: (date, lang) => formatDate(date, lang, 'LLLL'),
});

function purgeCache() {
    const hLen = hashes.length;
    let h;
    let l;
    let langsLen;

    for (h = 0; h < hLen; h += 1) {
        langsLen = cached[hashes[h]].length;
        for (l = 0; l < langsLen; l += 1) {
            if (cached[hashes[h]][l].lastRequested !== undefined
                && new Date().getTime() - cached[hashes[h]][l].lastRequested > CACHE_EXPIRE_MS
            ) {
                delete cached[hashes[h]][l];
            }
        }
    }
}

function getTranslation(localesData, tHash, text, lang, type, extra, templateData = '') {
    let translation;
    let compiled;

    if (localesData.length) {
        if (!localesData[0].strings.length) {
            logger.warning(`Still missing translation for "${text}" (${lang})`);
        }
        compiled = localesData[0].strings[0].text;
    } else {
        // Insert where did we found the template as extra data
        missing({ text, lang, type, extra });
        compiled = text;
    }

    hashes.push(tHash);
    cached[tHash] = cached[tHash] || {};
    cached[tHash][lang] = cached[tHash][lang] || {};
    cachedCount += 1;

    if (/\{\{/g.test(compiled) && templateData) {
        try {
            compiled = handlebars.compile(compiled);
            translation = compiled(templateData);
            cached[tHash][lang].translation = compiled;
            cached[tHash][lang].isCompiled = true;
        } catch (e) {
            logger.error('Handlebars error (t function):', text, lang, e);
            translation = text;
        }
    } else {
        cached[tHash][lang].isCompiled = false;
        cached[tHash][lang].translation = compiled;
        translation = compiled;
    }

    cached[tHash][lang].lastRequested = new Date().getTime();
    if (cachedCount > CACHE_MAX_KEYS) {
        setTimeout(purgeCache, 1);
    }

    return translation;
}

/**
 * Translates text and inserts in database automatically if it does not exist.
 *
 * @param text {String} The text to be translated
 * @param options {Object} an object with {
 *  lang: the desired language,
 *  type: type of translation 'client', 'server', etc... (server by default),
 *  extra: An extra string to identify where the translation comes from,
 *  templateData: If translation is a handlebars template, data to be use with the template
 * }
 * @return {Promise}
 */
function t(text, options) {
    const { lang, type, extra, templateData } = options;
    const tHash = hashCode(text);
    let promise;
    let sCallerName;
    {
        // See http://stackoverflow.com/a/36152335/513570
        const re = /(\w+)@|at (\w+) \(/g;
        const aRegexResult = re.exec(new Error().stack);
        sCallerName = aRegexResult[1] || aRegexResult[2];
    }

    if (cached[tHash] === undefined || cached[tHash][lang] === undefined) {
        promise = Locale
            .aggregate([
                { $match: { 'strings.text': text } },
                {
                    $project: {
                        strings: {
                            $filter: {
                                input: '$strings',
                                as: 'string',
                                cond: { $eq: ['$$string.lang', lang] },
                            },
                        },
                    },
                },
            ])
            .exec()
            .then(localesData => getTranslation(localesData, tHash, text, lang, type || 'server', extra || sCallerName, templateData));
    } else if (cached[tHash][lang].isCompiled) {
        promise = Promise.resolve(cached[tHash][lang].translation(templateData));
        cached[tHash][lang].lastRequested = new Date().getTime();
    } else {
        promise = Promise.resolve(cached[tHash][lang].translation);
        cached[tHash][lang].lastRequested = new Date().getTime();
    }

    return promise;
}

/**
 * Translates text and inserts in database automatically if it does not exist.
 *
 * @param text {Object} The text (string) or texts (array) to be translated
 * @param options {Object} an object with {
 *  lang: the desired language,
 *  type: type of translation 'client', 'server', etc... (server by default),
 *  extra: An extra string to identify where the translation comes from,
 *  templateData: If translation is a handlebars template, data to be use with the template
 * }
 */
export default (text, options) => {
    let promise;

    if (typeof text === 'string') {
        promise = t(text, options);
    } else {
        const promises = [];
        Object.keys(text).forEach(i => promises.push(t(text[i], options)));
        promise = Promise.all(promises);
    }
    return promise;
};
