/* eslint-disable no-unused-expressions */
// import request from 'supertest';
import sinon from 'sinon';
import { expect } from 'chai';

import nodeLangoose from '../';
import { addNewLanguage, getAvailableLangs, getLogger } from '../lib/lang-mongoose';
import { languages, stubLangFind, stubLangFindErr, stubLangfindOneAndUpdate } from './mongoMocks';

const LANG_MAX_AGE = 100;

describe('Initializing node-langoose', () => {
    stubLangFind();
    it('Init correctly and default logger works', () => {
        nodeLangoose();
        getLogger().log('Hello!');
        getLogger().error('Hello!');
        getLogger().warning('Hello!');
        getLogger().info('Hello!');
    });

    it('Logger should work', () => {
        const logger = {
            log: sinon.spy(),
        };
        nodeLangoose({
            logger,
        });
        getLogger().log('Hello!');
        sinon.assert.calledOnce(logger.log);
        sinon.assert.calledWith(logger.log, 'Hello!');
    });

    it('Get available languages should return languages', (done) => {
        getAvailableLangs()
            .then((langs) => {
                expect(langs).to.equal(languages);
                done();
            })
            .catch(done);
    });

    it('Email initialization should work', () => {
        nodeLangoose({
            email: {
                transport: {
                    sendMail: () => { },
                },
                from: '',
                to: '',
            },
        });
    });
});

describe('Select language', () => {
    const findStub = stubLangFind();
    it('Select (library default: en) default language and set "lang" cookie, no "accept-language" header in request', (done) => {
        const cookie = sinon.spy();
        const req = { headers: {} };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('en');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'en', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select (library default: en) default language and not set "lang" cookie, (ca) available, no "accept-language" header in request', (done) => {
        const cookie = sinon.spy();
        const req = { headers: {} };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
            isSetCookie: false,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('en');
                    sinon.assert.notCalled(cookie);
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select (ca) default language and set "lang" cookie, (ca) available, no "accept-language" header in request', (done) => {
        const cookie = sinon.spy();
        const req = { headers: {} };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
            defaultLang: 'ca',
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('ca');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'ca', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select language using "accept-language" header, (ca) first option, available', (done) => {
        const cookie = sinon.spy();
        const req = {
            headers: {
                'accept-language': 'ca,en-UK;q=0.8,fr;q=0.6,es-ES;q=0.4,es;q=0.2',
            },
        };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('ca');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'ca', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select language using "accept-language" header, (fr) first option not available, (ca) second option, available', (done) => {
        const cookie = sinon.spy();
        const req = {
            headers: {
                'accept-language': 'fr,ca;q=0.8,en-UK;q=0.8,es-ES;q=0.4,es;q=0.2',
            },
        };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('ca');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'ca', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select language using "accept-language" header, (en-UK) first option not available, (ca) second option, available', (done) => {
        const cookie = sinon.spy();
        const req = {
            headers: {
                'accept-language': 'en-UK,ca;q=0.8,fr;q=0.6,es-ES;q=0.4',
            },
        };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('ca');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'ca', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select language using "accept-language" header, (en-UK) first option not available, (en) generic available', (done) => {
        const cookie = sinon.spy();
        const req = {
            headers: {
                'accept-language': 'en-UK;q=0.8,fr;q=0.6,es-ES;q=0.4',
            },
        };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('en');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'en', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select language (library default: en) using "accept-language" header, none of them available', (done) => {
        const cookie = sinon.spy();
        const req = {
            headers: {
                'accept-language': 'fr,ru',
            },
        };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('en');
                    sinon.assert.calledOnce(cookie);
                    sinon.assert.calledWithExactly(cookie, 'lang', 'en', {
                        maxAge: LANG_MAX_AGE,
                        httpOnly: false,
                    });
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });

    it('Select (en) from cookie, don\'t call set cookie', (done) => {
        const cookie = sinon.spy();
        const req = {
            headers: {
                'accept-language': 'en-UK;q=0.8,fr;q=0.6,es-ES;q=0.4',
            },
            cookies: {
                lang: 'en',
            },
        };
        const res = { cookie };
        const middleWare = nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                middleWare(req, res, (err) => {
                    expect(err).to.be.undefined;
                    sinon.assert.calledOnce(findStub.stub);
                    expect(req).to.have.property('lang').and.equal('en');
                    sinon.assert.notCalled(cookie);
                    cookie.reset();
                    done();
                });
            })
                .catch(done);
        });
    });
});

describe('Add new language', () => {
    stubLangFind();
    const findOneAndUpdate = stubLangfindOneAndUpdate();

    it('Create a new language in database (fr)', (done) => {
        nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                addNewLanguage('fr');
                sinon.assert.calledOnce(findOneAndUpdate.stub);
                sinon.assert.calledWithExactly(findOneAndUpdate.stub, { lang: 'fr' }, { lang: 'fr', displayName: '' }, { new: true, upsert: true });
                done();
            })
                .catch(done);
        });
    });

    it('Do not create existing language (ca)', (done) => {
        nodeLangoose({
            maxAge: LANG_MAX_AGE,
        }, (p) => {
            p.then(() => {
                addNewLanguage('ca');
                sinon.assert.notCalled(findOneAndUpdate.stub);
                done();
            })
                .catch(done);
        });
    });
});

describe('Database error', () => {
    const findErr = stubLangFindErr();

    it('Log a database error', (done) => {
        const logger = {
            error: sinon.spy(),
        };
        nodeLangoose({
            logger,
        }, (p) => {
            p
                .then(() => done('Promise should not be resolved'))
                .catch((err) => {
                    expect(err).to.equal('Database error');
                    sinon.assert.calledOnce(findErr.stub);
                    sinon.assert.calledOnce(logger.error);
                    sinon.assert.calledWithExactly(logger.error, 'Error selecting language:', 'Database error');
                    done();
                })
                .catch(done);
        });
    });
});
